/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "pingcheck.h"

#include <QProcess>

#include <KDebug>

#include "serverstatus.h"

namespace ServerStatus
{

class PingCheckPrivate
{
public:
	QString host;
	QString program;
	QStringList args;
	uint pings;
	
	static Status statusUp;
	static Status statusDown;
	static Status statusError;
	
	int result;
};

} // end namespace ServerStatus


using namespace ServerStatus;

Ui::PingCheckSettings PingCheck::m_settingsUi;

Status PingCheckPrivate::statusUp(Status::Up, 
								  i18nc("server status", "up"), 
								  i18n("The server is up."));
Status PingCheckPrivate::statusDown(Status::Down, 
									i18nc("server status", "down"), 
									i18n("No ping replies received!"));
Status PingCheckPrivate::statusError(Status::Error, 
									 i18nc("server status", "check failed"), 
									 i18n("Failed to execute the ping process."));

PingCheck::PingCheck() : Check(Check::TypePing)
{
	d = new PingCheckPrivate;
	d->result = 0;
	
	// TODO Should this be configurable?
	d->program = "/bin/ping";
}

PingCheck::~PingCheck()
{
	delete d;
}

// static
void PingCheck::showSettings(QWidget* w, const Check* const currentCheck)
{
	PingCheck::m_settingsUi.setupUi(w);
	if (currentCheck != 0L && currentCheck->type() == Check::TypePing) {
		QStringList params = currentCheck->parameters();
		PingCheck::m_settingsUi.leHost->setText(params.at(0));
		PingCheck::m_settingsUi.sbPings->setValue(params.at(1).toUInt());
	}
}

void PingCheck::applySettings()
{
	d->host = PingCheck::m_settingsUi.leHost->text().trimmed();
	d->pings = PingCheck::m_settingsUi.sbPings->value();
}

QStringList PingCheck::parameters() const
{
	QStringList result;
	result << d->host;
	result << QString::number(d->pings);
	return result;
}

void PingCheck::setParameters(const QStringList& parameters)
{
	if (parameters.size() == 2) {
		d->host = parameters.at(0);
		d->pings = parameters.at(1).toUInt();
	}
}

const Status& PingCheck::result()
{
	switch (d->result) {
	case 1:
		return PingCheckPrivate::statusUp;
	case 2:
		return PingCheckPrivate::statusDown;
	case 3:
		return PingCheckPrivate::statusError;
	default:
		return Status::None;
	}
}

void PingCheck::run()
{
	kDebug(0) << "Starting new thread for ping check";
	QProcess ping;
	QStringList args;
	
	args << d->host << "-c" << QString::number(d->pings);
	
	ping.start(d->program, args);
	ping.waitForFinished();
	
	if (ping.exitStatus() == QProcess::CrashExit) {
		d->result = 3;
		return;
	}

	switch (ping.exitCode()) {
	case 0:
		// at least one packet received
		d->result = 1;
		return;
	case 1:
		// pings sent, but no replies received
		d->result = 2;
		return;
	default:
		// usually 2, means some error occured (e.g. malformed host)
		d->result = 3;
		return;
	}
}

#include "pingcheck.moc"
