/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "server.h"

#include <QString>
#include <QTimer>

#include <KDebug>
#include <KConfigGroup>

#include "check.h"
#include "serverstatus.h"

using namespace std;
using namespace ServerStatus;

namespace ServerStatus
{

class ServerPrivate
{
public:
	QString name;
	QTimer timer;
	QString checkType;
	QString checkConfig;
	Check* check;
	const Status* status;
	bool active;
	bool slatedForRemoval;
};
	
} // end namespace ServerStatus


Server::Server(const QString& id) :
	QObject(),
	m_id(id),
	m_cfgTemplate("server_" + id + "_%1")
{
	d = new ServerPrivate;
	d->check = 0L;
	d->status = 0L;
	d->slatedForRemoval = false;
}

Server::Server(const QString& id, KConfigGroup& config)	: 
	m_id(id),
	m_cfgTemplate("server_" + id + "_%1")
{
	d = new ServerPrivate;
	d->slatedForRemoval = false;
	readConfig(config);
}

Server::~Server()
{
	if (d->check != 0) {
		d->check->deleteLater();
	}
	delete d;
}

void Server::readConfig(KConfigGroup& config)
{
	d->name = config.readEntry(m_cfgTemplate.arg("name"), QString());
	d->active = config.readEntry(m_cfgTemplate.arg("active"), true);
	setInterval(config.readEntry(m_cfgTemplate.arg("interval"), 60));
	QString checkType = config.readEntry(m_cfgTemplate.arg("checkType"), QString());
	QStringList checkConfig = config.readEntry(m_cfgTemplate.arg("checkConfig"), QStringList());
	
	kDebug(0) << "Loaded server" << name();
	if (!checkType.isEmpty()) {
		Check* check = Check::createCheck(checkType);
		check->setParameters(checkConfig);
		setCheck(check);
	}
}

void Server::saveConfig(KConfigGroup& config)
{
	config.writeEntry(m_cfgTemplate.arg("name"), d->name);
	config.writeEntry(m_cfgTemplate.arg("active"), d->active);
	config.writeEntry(m_cfgTemplate.arg("interval"), interval());
	config.writeEntry(m_cfgTemplate.arg("checkType"), d->check->type());
	config.writeEntry(m_cfgTemplate.arg("checkConfig"), d->check->parameters());
}

void Server::removeConfig(KConfigGroup& config)
{
	config.deleteEntry(m_cfgTemplate.arg("name"));
	config.deleteEntry(m_cfgTemplate.arg("active"));
	config.deleteEntry(m_cfgTemplate.arg("interval"));
	config.deleteEntry(m_cfgTemplate.arg("checkType"));
	config.deleteEntry(m_cfgTemplate.arg("checkConfig"));
}

void Server::slotTimeout()
{
	if (d->active) {
		d->check->start();
	}
}

void Server::slotCheckFinished()
{
	if (d->slatedForRemoval) {
		delete this;
	}
	else if (&d->check->result() != d->status) {
		kDebug(0) << "Emitting status changed (" << d->status << "->" << &d->check->result() << ")";

		// This signal is for the notifications. Only emit if the status was set
		// before to avoid sending "Server up" notifications on startup.
		if (d->status != 0L) {
			emit(statusChanged(this, d->check->result()));
		}
		
		d->status = &d->check->result();
		
		// This signal is for the icon and must always be emitted.
		emit(statusChanged(d->check->result()));
	}
}

bool Server::active() const
{
	return d->active;
}

void Server::setActive(bool active)
{
	d->active = active;
}

void Server::startChecks()
{
	// Do one check immediately, then start timer.
	slotTimeout();
	d->timer.start();
}

void Server::stopChecks()
{
	d->timer.stop();
}

bool Server::checksRunning()
{
	return d->timer.isActive();
}

const QString& Server::name() const
{
	return d->name;
}

void Server::setName(const QString& name)
{
	d->name = name;
}

int Server::interval() const {
	return d->timer.interval() / 1000;
}

void Server::setInterval(int interval) {
	d->timer.setInterval(interval * 1000);
}

Check* Server::check()
{
	return d->check;
}

void Server::setCheck(Check* check)
{
	// TODO check if thread is running?
	stopChecks();
	if (d->check != 0L) {
		delete d->check;
	}
	d->check = check;
	// Reset the status to not trigger a "Server up" notification when the 
	// server ist first checked.
	d->status = 0L;
	connect(&d->timer, SIGNAL(timeout()), check, SLOT(start()));
	connect(d->check, SIGNAL(finished()), this, SLOT(slotCheckFinished()));
}

void Server::delayedDeletion()
{
	if (d->check != 0L && d->check->isRunning()) {
		d->slatedForRemoval = true;
	}
	else {
		delete this;
	}
}

void Server::forcedDeletion()
{
	if (d->check != 0L && d->check->isRunning()) {
		d->check->terminate();
	}
	delete this;
}

