/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "serverstatusapplet.h"

#include <QByteArray>
#include <QGraphicsLayoutItem>
#include <QGraphicsGridLayout>
#include <QLabel>
#include <QPainter>
#include <QRect>
#include <QRegExp>
#include <QTimer>

#include <KAboutData>
#include <KConfigDialog>
#include <KDebug>
#include <KGlobal>
#include <KNotification>
#include <KNotifyConfigWidget>

#include <Plasma/Label>

#include "check.h"
#include "server.h"
#include "servericon.h"
#include "serverstatus.h"
#include "serverstatusconfig.h"


namespace ServerStatus
{

class AppletPrivate
{
public:
	
	QList<QGraphicsItem*> widgets;

	Config *config;
	
	KComponentData *compData;
};

}

using namespace ServerStatus;

Applet::Applet(QObject *parent, const QVariantList &args) : Plasma::Applet(parent, args)
{
	d = new AppletPrivate;
	
	// We need some custom AboutData to make notifications work correctly.
	KAboutData aboutData("plasma-applet-serverstatus", 0, ki18n("Server Status Widget"),
		"1.4", ki18n("Plasma applet to monitor remote hosts"), KAboutData::License_GPL,
		ki18n("Copyright (c) 2009-2011, Conrad Hoffmann"), KLocalizedString(), "http://code.bitfehler.net");
	d->compData = new KComponentData(aboutData);
	
	// This will get us the standard applet background, for free!
	setBackgroundHints(DefaultBackground);
	
	// Allow free resizing (i.e. must not be a square).
	setAspectRatioMode(Plasma::IgnoreAspectRatio);

	// Set a sane default size if the widget starts for the first time. If it's
	// not the first time, the size is set automatically before init().
	resize(200, 50);
}

Applet::~Applet()
{
	this->hide();
	delete d->compData;
	delete d->config;
	delete d;
}

void Applet::init()
{
	KConfigGroup cfg = config();
	d->config = new Config(this);
	d->config->loadConfig(cfg);
	connect(d->config, SIGNAL(configChanged()), this, SLOT(slotConfigChanged()));
	
	buildLayout();

	foreach (Server* server, d->config->servers()) {
		server->startChecks();
	}
}

void Applet::buildLayout()
{
	setFont(d->config->serverFont());
	
	// Remove and delete all currently displayed items. There may be a more 
	// decent way to do this, but they have to be hidden _and_ removed.
	QGraphicsGridLayout* layout = 0L;
	if (this->layout() == 0L) {
		layout = new QGraphicsGridLayout();
		setLayout(layout);
	}
	else {
		layout = dynamic_cast<QGraphicsGridLayout*>(this->layout());
	}
	for (int i = d->widgets.size() - 1; i >= 0; --i) {
		QGraphicsItem* w = dynamic_cast<QGraphicsItem*>(layout->itemAt(i));
		if (w != 0L) {
			layout->removeAt(i);
			w->hide();
			delete w;
		}
	}
	d->widgets.clear();
		
	QList<Server*> servers = d->config->servers();

	if (!servers.empty()) {

		for (int i = 0; i < servers.size(); ++i) {
			Server* server = servers.at(i);
			
			ServerIcon* icon = new ServerIcon(d->config, this);
			icon->setIcon(d->config->iconForStatus(server->check()->result()));
			connect(server, SIGNAL(statusChanged(const ServerStatus::Status&)),
					icon, SLOT(slotChangeStatus(const ServerStatus::Status&)));
			disconnect(server, 0, this, 0);
			connect(server, SIGNAL(statusChanged(const ServerStatus::Server*, const ServerStatus::Status&)),
					this, SLOT(slotServerStatusChanged(const ServerStatus::Server*, const ServerStatus::Status&)));
			layout->addItem(icon, i, 0);
			d->widgets.append(icon);
			
			Plasma::Label* label = new Plasma::Label;
			label->setText(server->name());
			
			// In some older versions of Qt, label->setFont did not work - thus also the call to set
			// the label of the native QLabel behind it. It may not be neccessary anymore, but it
			// can't harm either.
			label->setFont(d->config->serverFont());
			label->nativeWidget()->setFont(d->config->serverFont());
			
			layout->addItem(label, i, 1);
			d->widgets.append(label);

			int labelWidth = label->nativeWidget()->minimumSizeHint().width();
			if ( labelWidth > (minimumWidth() - 50)) {
				// The 50 is for the icon and borders.
				setMinimumWidth(labelWidth + 50);
			}
			setMinimumHeight(label->nativeWidget()->minimumSizeHint().height() * servers.size() + 40);
		}
	}
	else {
		Plasma::Label* label = new Plasma::Label;
		label->setText(i18n("No servers configured"));
		layout->addItem(label, 0, 0);
		d->widgets.append(label);
		setMinimumWidth(label->nativeWidget()->minimumSizeHint().width() + 20);
		setMinimumWidth(label->nativeWidget()->minimumSizeHint().height() + 20);
	}

	layout->invalidate();
	update();
	emit(appletTransformedItself());
}

void Applet::createConfigurationInterface(KConfigDialog *parent)
{
	// Let the Config object handle most of the stuff, except for...
    d->config->createConfigurationInterface(parent);
	
	// ... the notifications part. This is easier to do right here.
	KNotifyConfigWidget *notifyWidget = new KNotifyConfigWidget(parent);
	notifyWidget->setApplication(d->compData->componentName());
	parent->addPage(notifyWidget, i18nc("@title:tab Preferences", "Notifications"), 
					"preferences-desktop-notification");
	connect(parent, SIGNAL(applyClicked()), notifyWidget, SLOT(save()));
    connect(parent, SIGNAL(okClicked()), notifyWidget, SLOT(save()));
}

void Applet::slotConfigChanged()
{
	KConfigGroup cfg = config();
	d->config->saveConfig(cfg);

	buildLayout();
}

void Applet::slotServerStatusChanged(const Server* server, const Status& status)
{
	kDebug() << "Notify:" << status.eventName();
	KNotification *notification= new KNotification(status.eventName());
	notification->setComponentData(*d->compData);
#if KDE_VERSION_MINOR > 2
	// This was added in KDE 4.3
	notification->setTitle(i18nc("@info Notification title", "Server status notification"));
#endif
	QString text = i18nc("@info:status Notification text", 
						 "<b>New status for server <i>%1</i>: <i>%2</i></b>", 
						 server->name(), status.name());
	text.append("<br><br>" + status.description());
	notification->setText(text);
	// Once the configuration for per server notifications is in place, this 
	// could be used to set the contexts.
// 	notification->addContext("server", server->name());
// 	notification->addContext("status", status.name());
	notification->sendEvent();
}


#include "serverstatusapplet.moc"
