/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "serverstatusconfig.h"

#include <QDateTime>
#include <QDialog>
#include <QFont>
#include <QInputDialog>
#include <QLayoutItem>
#include <QListWidgetItem>
#include <QMap>

#include <KConfigDialog>
#include <KDebug>
#include <KIcon>
#include <KIconDialog>
#include <KFontDialog>

#include "check.h"
#include "server.h"
#include "serverstatus.h"
#include "ui_generalsettings.h"
#include "ui_appearancesettings.h"
#include "ui_serversettings.h"


namespace ServerStatus
{

class ConfigPrivate
{
public:
	QStringList serverIds;
	QList<Server*> servers;
	QList<Server*> tmpServers;
	QString iconOk;
	QString tmpIconOk;
	QString iconError;
	QString tmpIconError;
	QString iconWarning;
	QString tmpIconWarning;
	QString iconUnknown;
	QString tmpIconUnknown;
	QFont font;
	
	int interval;
	int pings;
	
	KConfigDialog* currentParent;
	Ui::ServerSettings serverSettingsUi;
	Ui::GeneralSettings generalSettingsUi;
	Ui::AppearanceSettings appearanceSettingsUi;
};
	
} // end namespace ServerStatus

using namespace ServerStatus;

Config::Config(QObject* parent)
	: QObject(parent)
{
	d = new ConfigPrivate;
}

Config::~Config()
{
	while (!d->servers.isEmpty()) {
		d->servers.takeFirst()->forcedDeletion();
	}
	delete d;
}

void Config::loadConfig(KConfigGroup& config)
{
	d->serverIds = config.readEntry("serverIds", QStringList());
	d->iconOk = config.readEntry("iconOk", "security-high");
	d->iconError = config.readEntry("iconError", "security-low");
	d->iconWarning = config.readEntry("iconWarning", "security-medium");
	d->iconUnknown = config.readEntry("iconUnknown", "view-refresh");
	d->font = config.readEntry("serverFont", QFont());
	
	// Load the servers
	foreach (const QString id, d->serverIds) {
		Server* server = new Server(id);
		server->readConfig(config);
		d->servers.append(server);
	}

	// The temporary variables must be initialized to the same values as the 
	// 'real' ones...
	d->tmpIconOk = d->iconOk;
	d->tmpIconError = d->iconError;
	d->tmpIconWarning = d->iconWarning;
	d->tmpIconUnknown = d->iconUnknown;
}

void Config::saveConfig(KConfigGroup& config)
{
	// Delete the config entries for all removed servers and the servers 
	// themselves.
	foreach (Server* server, d->servers) {
		if (!d->tmpServers.contains(server)) {
			server->removeConfig(config);
			server->delayedDeletion();
		}
	}

	d->servers = d->tmpServers;
	
	// Rebuild the ID list from the actual server map.
	d->serverIds.clear();
	foreach (Server* server, d->servers) {
		d->serverIds.append(server->id());
	}
	
	config.writeEntry("serverIds", d->serverIds);
	config.writeEntry("iconOk", d->iconOk);
	config.writeEntry("iconError", d->iconError);
	config.writeEntry("iconWarning", d->iconWarning);
	config.writeEntry("iconUnknown", d->iconUnknown);
	config.writeEntry("serverFont", d->font);	

	foreach (Server* server, d->servers) {
		server->saveConfig(config);
	}

	reloadServerList();
}

void Config::createConfigurationInterface(KConfigDialog* parent)
{
	// Remember the parent dialog for later. This seems hacky, but it is important
	// in order to make the server editing dialogs window modal (which has no
	// effect if they are not a child of this dialog). Showing application modal
	// dialogs makes all of Plasma unusable while the dialog is shown, including
	// the task bar and all panels.
	d->currentParent = parent;
	
	// General settings page
    QWidget* widget = new QWidget(parent);
    d->generalSettingsUi.setupUi(widget);
	d->generalSettingsUi.btAddServer->setIcon(KIcon("list-add"));
	d->generalSettingsUi.btEditServer->setIcon(KIcon("configure"));
	d->generalSettingsUi.btRemoveServer->setIcon(KIcon("list-remove"));
	d->generalSettingsUi.btMoveUp->setIcon(KIcon("go-up"));
	d->generalSettingsUi.btMoveDown->setIcon(KIcon("go-down"));
	
	// The config operates on a copy of the server list, so things can be undone
	// if the user cancels the dialog, which is kind of the expected behaviour.
	d->tmpServers = d->servers;
	reloadServerList();
	
	connect(d->generalSettingsUi.serverList, SIGNAL(itemSelectionChanged()), this, SLOT(slotServerListSelectionChanged()));
	connect(d->generalSettingsUi.btAddServer, SIGNAL(clicked()), this, SLOT(slotAddServer()));
	connect(d->generalSettingsUi.btEditServer, SIGNAL(clicked()), this, SLOT(slotEditServer()));
	connect(d->generalSettingsUi.serverList, SIGNAL(doubleClicked(const QModelIndex&)), this, SLOT(slotEditServer()));
	connect(d->generalSettingsUi.btRemoveServer, SIGNAL(clicked()), this, SLOT(slotRemoveServer()));
	connect(d->generalSettingsUi.btMoveUp, SIGNAL(clicked()), this, SLOT(slotMoveServerUp()));
	connect(d->generalSettingsUi.btMoveDown, SIGNAL(clicked()), this, SLOT(slotMoveServerDown()));
    
	parent->addPage(widget, i18nc("@title:tab Preferences", "General"), "preferences-other");
	
	// Appearance settings page
	widget = new QWidget();
    d->appearanceSettingsUi.setupUi(widget);
	d->appearanceSettingsUi.btSelectIconOk->setIcon(KIcon(d->iconOk));
	d->appearanceSettingsUi.btSelectIconError->setIcon(KIcon(d->iconError));
	d->appearanceSettingsUi.btSelectIconWarning->setIcon(KIcon(d->iconWarning));
	d->appearanceSettingsUi.btSelectIconUnknown->setIcon(KIcon(d->iconUnknown));
	
	connect(d->appearanceSettingsUi.btSelectIconOk, SIGNAL(clicked()), this, SLOT(slotSelectIconOk()));
	connect(d->appearanceSettingsUi.btSelectIconError, SIGNAL(clicked()), this, SLOT(slotSelectIconError()));
	connect(d->appearanceSettingsUi.btSelectIconWarning, SIGNAL(clicked()), this, SLOT(slotSelectIconWarning()));
	connect(d->appearanceSettingsUi.btSelectIconUnknown, SIGNAL(clicked()), this, SLOT(slotSelectIconUnknown()));
	
	d->appearanceSettingsUi.btSelectServerFont->setText(d->font.family());
	d->appearanceSettingsUi.btSelectServerFont->setFont(d->font);
	
	connect(d->appearanceSettingsUi.btSelectServerFont, SIGNAL(clicked()), this, SLOT(slotSelectServerFont()));
	
	parent->addPage(widget, i18nc("@title:tab Preferences", "Appearance"), "format-fill-color");
	
	// Connect to the parent dialogs buttons
	connect(parent, SIGNAL(cancelClicked()), this, SLOT(slotConfigCancelled()));
	connect(parent, SIGNAL(applyClicked()), this, SLOT(slotConfigAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(slotConfigAccepted()));
}

void Config::reloadServerList()
{
	d->generalSettingsUi.serverList->clear();
	
	// Show the current servers.
	foreach(Server* server, d->tmpServers) {
		// Create a list item with the name of the Server and set the ID of the 
		// Server as user data, so it can be identified in case of ambigious 
		// names.
		QListWidgetItem* item = new QListWidgetItem(server->name());
		item->setData(Qt::UserRole, server->id());
		item->setToolTip("Server Id: " + server->id());
		d->generalSettingsUi.serverList->addItem(item);
	}
}

void Config::slotServerListSelectionChanged()
{
	// This is mostly UI sugar. Enable only the buttons whose actions make sense
	// for the current selection (if any).
	bool enabled = d->generalSettingsUi.serverList->selectedItems().size() > 0;
	int currentRow = d->generalSettingsUi.serverList->currentRow();
	int maxRow = d->tmpServers.size() - 1;
	
	d->generalSettingsUi.btEditServer->setEnabled(enabled);
	d->generalSettingsUi.btRemoveServer->setEnabled(enabled);
	d->generalSettingsUi.btMoveUp->setEnabled(enabled && currentRow > 0);
	d->generalSettingsUi.btMoveDown->setEnabled(enabled && currentRow < maxRow);
}

void Config::slotCheckTypeSelectionChanged(const QString& name)
{
	// Whenever the user changes the type of check for a server while editing a
	// server, this will setup the appropriate configuration GUI in a part of 
	// the dialog.
	
	// Remove the current GUI elements, if there are any. If this called the first time the dialog
	// is shown, there won't be a layout yet.
	if (d->serverSettingsUi.gbCheckSettings->layout() != 0L) {
		QLayoutItem* item;
		while ((item = d->serverSettingsUi.gbCheckSettings->layout()->takeAt(0)) != 0L) {
			delete item;
		}
		delete d->serverSettingsUi.gbCheckSettings->layout();
		foreach(QObject* o, d->serverSettingsUi.gbCheckSettings->children()) {
			o->deleteLater();
		}
	}
	
	// Check if a server is selected and if it already has a check assigned.
	Check* check = 0L;
	if (d->generalSettingsUi.serverList->currentRow() >= 0) {
		check = d->tmpServers.at(d->generalSettingsUi.serverList->currentRow())->check();
	}
	
	// This sets up the new GUI elements. If the server has a check assigned, 
	// its values are used as defaults, but obviously only for same type.
	Check::showSettings(d->serverSettingsUi.gbCheckSettings, name, check);
}

void Config::slotAddServer()
{
	// This is important so that slotCheckTypeSelectionChanged() knows we are
	// not editing an existing server.
	d->generalSettingsUi.serverList->setCurrentRow(-1);
	
	QDialog addServerDialog(d->currentParent);
	addServerDialog.setWindowModality(Qt::WindowModal);
	d->serverSettingsUi.setupUi(&addServerDialog);
	d->serverSettingsUi.cbCheckType->insertItems(0, Check::checkTypes());
	
	connect(d->serverSettingsUi.cbCheckType, SIGNAL(currentIndexChanged(const QString&)),
			this, SLOT(slotCheckTypeSelectionChanged(const QString&)));
	// Trigger once to have the initial settings displayed
	slotCheckTypeSelectionChanged(Check::TypePing);
	
	if (addServerDialog.exec() == QDialog::Accepted) {
		QString id = QString::number(QDateTime::currentDateTime().toTime_t());
		Server* server = new Server(id);
		
		server->setName(d->serverSettingsUi.leName->text());
		server->setActive(d->serverSettingsUi.cbActive->isChecked());
		server->setInterval(d->serverSettingsUi.sbInterval->value());

		Check* check = Check::createCheck(d->serverSettingsUi.cbCheckType->currentText());
		check->applySettings();
		server->setCheck(check);
		d->tmpServers.append(server);
		reloadServerList();
	}
}

void Config::slotEditServer()
{
	Server* server = d->tmpServers.at(d->generalSettingsUi.serverList->currentRow());
	
	QDialog editServerDialog(d->currentParent);
	editServerDialog.setWindowModality(Qt::WindowModal);
	d->serverSettingsUi.setupUi(&editServerDialog);
	d->serverSettingsUi.cbCheckType->insertItems(0, Check::checkTypes());
	
	d->serverSettingsUi.leName->setText(server->name());
	d->serverSettingsUi.cbActive->setChecked(server->active());
	d->serverSettingsUi.sbInterval->setValue(server->interval());
	int index = d->serverSettingsUi.cbCheckType->findText(server->check()->type());
	d->serverSettingsUi.cbCheckType->setCurrentIndex(index);
	
	connect(d->serverSettingsUi.cbCheckType, SIGNAL(currentIndexChanged(const QString&)),
			this, SLOT(slotCheckTypeSelectionChanged(const QString&)));
	// Trigger once to have the initial settings displayed
	slotCheckTypeSelectionChanged(server->check()->type());

	
	if (editServerDialog.exec() == QDialog::Accepted) {
		server->setName(d->serverSettingsUi.leName->text());
		server->setActive(d->serverSettingsUi.cbActive->isChecked());
		server->setInterval(d->serverSettingsUi.sbInterval->value());

		if (d->serverSettingsUi.cbCheckType->currentText() != server->check()->type()) {
			Check* check = Check::createCheck(d->serverSettingsUi.cbCheckType->currentText());
			server->setCheck(check);
		}

		server->check()->applySettings();
		reloadServerList();
	}
}

void Config::slotRemoveServer()
{
	d->tmpServers.removeAt(d->generalSettingsUi.serverList->currentRow());
	reloadServerList();
}

void Config::slotMoveServerUp()
{
	// Check that the server can actually be moved up. The button should not be
	// enabled unless it can, but better safe than sorry.
	int cur = d->generalSettingsUi.serverList->currentRow();
	if (cur > 0) {
		d->tmpServers.swap(cur, cur - 1);
	}
	reloadServerList();
	d->generalSettingsUi.serverList->setCurrentRow(cur - 1);
}

void Config::slotMoveServerDown()
{
	// Check that the server can actually be moved down. See above.
	int cur = d->generalSettingsUi.serverList->currentRow();
	if (cur < d->tmpServers.size() - 1) {
		d->tmpServers.swap(cur, cur + 1);
	}
	reloadServerList();
	d->generalSettingsUi.serverList->setCurrentRow(cur + 1);	
}

void Config::slotSelectIconOk()
{
	// A note on all of dialogs (icon and font) displayed in this configuration:
	// There are more convenient ways to use these (namely static functions),
	// but they all show the dialogs application modal. As already explained 
	// elsewhere, this will cause all of Plasma (Plasmoids, taskbar, panels) to
	// become unusable while the dialog is displayed. To avoid this, the dialogs
	// are shown window modal (only blocking the parent window), which takes
	// two lines of code more, but is absolutely neccessary in this case.
	KIconDialog dialog(KIconLoader::global(), d->currentParent);
	dialog.setWindowModality(Qt::WindowModal);
	QString icon = dialog.openDialog();
	if (!icon.isEmpty()) {
		d->tmpIconOk = icon;
		d->appearanceSettingsUi.btSelectIconOk->setIcon(KIcon(d->tmpIconOk));
	}
}

void Config::slotSelectIconError()
{
	// See comment in slotSelectIconOk().
	KIconDialog dialog(KIconLoader::global(), d->currentParent);
	dialog.setWindowModality(Qt::WindowModal);
	QString icon = dialog.openDialog();
	if (!icon.isEmpty()) {
		d->tmpIconError = icon;
		d->appearanceSettingsUi.btSelectIconError->setIcon(KIcon(d->tmpIconError));
	}
}

void Config::slotSelectIconWarning()
{
	// See comment in slotSelectIconOk().
	KIconDialog dialog(KIconLoader::global(), d->currentParent);
	dialog.setWindowModality(Qt::WindowModal);
	QString icon = dialog.openDialog();
	if (!icon.isEmpty()) {
		d->tmpIconWarning = icon;
		d->appearanceSettingsUi.btSelectIconWarning->setIcon(KIcon(d->tmpIconWarning));
	}
}

void Config::slotSelectIconUnknown()
{
	// See comment in slotSelectIconOk().
	KIconDialog dialog(KIconLoader::global(), d->currentParent);
	dialog.setWindowModality(Qt::WindowModal);
	QString icon = dialog.openDialog();
	if (!icon.isEmpty()) {
		d->tmpIconUnknown = icon;
		d->appearanceSettingsUi.btSelectIconUnknown->setIcon(KIcon(d->tmpIconUnknown));
	}
}

void Config::slotSelectServerFont()
{
	// See comment in slotSelectIconOk().
	KFontDialog dialog(d->currentParent);
	dialog.setWindowModality(Qt::WindowModal);
	dialog.setFont(d->appearanceSettingsUi.btSelectServerFont->font());
	int result = dialog.exec();
	if (result == KFontDialog::Accepted) {
		d->appearanceSettingsUi.btSelectServerFont->setText(dialog.font().family());
		d->appearanceSettingsUi.btSelectServerFont->setFont(dialog.font());
	}
}
	
void Config::slotConfigAccepted()
{
	// Transfer all temporary values to the real ones.
	d->iconOk = d->tmpIconOk;
	d->iconError = d->tmpIconError;
	d->iconWarning = d->tmpIconWarning;
	d->iconUnknown = d->tmpIconUnknown;
	d->font = d->appearanceSettingsUi.btSelectServerFont->font();

	// Stop checking for all removed servers. The lists 'servers' and 
	// 'tmpServers' are synced when the config is written, so that the 
	// configuration for these servers can be removed.
	foreach (Server* server, d->servers) {
		if (!d->tmpServers.contains(server)) {
			server->stopChecks();
		}
	}

	// Start checks for the new servers and servers for which the check type 
	// has changed (which causes the checks to stop).
	foreach (Server* server, d->tmpServers) {
		if (!d->servers.contains(server) || !server->checksRunning()) {
			server->startChecks();
		}
	}
	
	// This also triggers a rebuild of the layout, but only after having called
	// saveConfig() of this class.
	emit(configChanged());
}

void Config::slotConfigCancelled()
{
	// Delete the newly added servers.
	while (!d->tmpServers.isEmpty()) {
		Server* server = d->tmpServers.takeFirst();
		if (!d->servers.contains(server)) {
			server->delayedDeletion();
		}
	}
	
	// Synchronize temporary and actual server list. This also restores deleted
	// servers.
	d->tmpServers = d->servers;
}

const QList<Server*>& Config::servers() const
{
	return d->servers; 
}

const QString& Config::iconForStatus(const Status &status) const
{
	switch (status.type()) {
		case Status::Up:
			return d->iconOk;
		case Status::Down:
			return d->iconError;
		case Status::Warning:
			return d->iconWarning;
		case Status::Error:
			return d->iconError;
		default:
			return d->iconUnknown;
	}
}

const QString& Config::iconOk()
{
	return d->iconOk;
}

const QString& Config::iconError()
{
	return d->iconError;
}

const QString& Config::iconWarning()
{
	return d->iconWarning;
}

const QString& Config::iconUnknown()
{
	return d->iconUnknown;
}

const QFont& Config::serverFont()
{
	return d->font;
}

#include "serverstatusconfig.moc"
